@@ -14,7 +14,7 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
 
 # Note this code was originally licensed under the GNU GPL instead of the
 # GNU LGPL, its license has been changed with permission, see the permission
@@ -110,8 +110,8 @@ static int fun_C(int *bitfill, int gkw)
 	if (gkw == 0xfe) {
 		if (nbits(bitfill, 1) == 0)
 			return 7;
-		else
-			return -8;
+
+		return -8;
 	}
 
 	if (gkw != 0xff)
