@@ -414,7 +414,7 @@ std::string ts2s(__u64 ts)
 		strftime(buf, sizeof(buf), "%a %b %e %T.000000", &tm);
 	}
 	secs = last_secs + t - last_t;
-	sprintf(buf + 14, "%02u:%02u.%06lu", secs / 60, secs % 60, res.tv_usec);
+	sprintf(buf + 14, "%02u:%02u.%06llu", secs / 60, secs % 60, (__u64)res.tv_usec);
 	return buf;
 }
 
@@ -942,10 +942,10 @@ static void monitor(const struct node &node, __u32 monitor_time, const char *sto
 		}
 		fprintf(fstore, "# cec-ctl --store-pin\n");
 		fprintf(fstore, "# version %d\n", CEC_CTL_VERSION);
-		fprintf(fstore, "# start_monotonic %lu.%09lu\n",
-			start_monotonic.tv_sec, start_monotonic.tv_nsec);
-		fprintf(fstore, "# start_timeofday %lu.%06lu\n",
-			start_timeofday.tv_sec, start_timeofday.tv_usec);
+		fprintf(fstore, "# start_monotonic %llu.%09llu\n",
+			(__u64)start_monotonic.tv_sec, (__u64)start_monotonic.tv_nsec);
+		fprintf(fstore, "# start_timeofday %llu.%06llu\n",
+			(__u64)start_timeofday.tv_sec, (__u64)start_timeofday.tv_usec);
 		fprintf(fstore, "# log_addr_mask 0x%04x\n", node.log_addr_mask);
 		fprintf(fstore, "# phys_addr %x.%x.%x.%x\n",
 			cec_phys_addr_exp(node.phys_addr));
@@ -984,10 +984,10 @@ static void monitor(const struct node &node, __u32 monitor_time, const char *sto
 			 */
 			clock_gettime(CLOCK_MONOTONIC, &start_monotonic);
 			gettimeofday(&start_timeofday, nullptr);
-			fprintf(fstore, "# start_monotonic %lu.%09lu\n",
-				start_monotonic.tv_sec, start_monotonic.tv_nsec);
-			fprintf(fstore, "# start_timeofday %lu.%06lu\n",
-				start_timeofday.tv_sec, start_timeofday.tv_usec);
+			fprintf(fstore, "# start_monotonic %llu.%09llu\n",
+				(__u64)start_monotonic.tv_sec, (__u64)start_monotonic.tv_nsec);
+			fprintf(fstore, "# start_timeofday %llu.%06llu\n",
+				(__u64)start_timeofday.tv_sec, (__u64)start_timeofday.tv_usec);
 			fflush(fstore);
 			start_minute = now;
 		}
@@ -1978,13 +1978,13 @@ static void stress_test_random_standby_wakeup_cycle(const struct node &node, uns
 
 		cec_msg_init(&msg, wakeup_la, CEC_LOG_ADDR_TV);
 		cec_msg_image_view_on(&msg);
-		for (int i = 0; i < 50; i++) {
+		for (int i = 0; i < 10; i++) {
 			ret = transmit_msg_retry(node, msg);
 			if (!ret)
 				break;
 			usleep(200000);
 		}
-		if (ret)
+		if (ret != ENONET)
 			printf("%s\n", strerror(ret));
 		else
 			printf("OK\n");
@@ -2000,12 +2000,16 @@ static void stress_test_random_standby_wakeup_cycle(const struct node &node, uns
 			printf("Transmit Give Device Power Status from LA %s: ", cec_la2s(from));
 			ret = transmit_msg_retry(node, msg);
 			if (!ret) {
-				__u8 pwr;
+				if (cec_msg_status_is_ok(&msg)) {
+					__u8 pwr;
 
-				cec_ops_report_power_status(&msg, &pwr);
-				printf("%s\n", power_status2s(pwr));
-				if (pwr == CEC_OP_POWER_STATUS_ON)
-					goto done;
+					cec_ops_report_power_status(&msg, &pwr);
+					printf("%s\n", power_status2s(pwr));
+					if (pwr == CEC_OP_POWER_STATUS_ON)
+						goto done;
+				} else {
+					printf("%s\n", cec_status2s(msg).c_str());
+				}
 			} else {
 				printf("%s\n", strerror(ret));
 			}
@@ -2032,6 +2036,8 @@ static void stress_test_random_standby_wakeup_cycle(const struct node &node, uns
 			ret = transmit_msg_retry(node, msg);
 			if (ret) {
 				printf("%s\n", strerror(ret));
+			} else if (!cec_msg_status_is_ok(&msg)) {
+				printf("%s\n", cec_status2s(msg).c_str());
 			} else {
 				__u8 pwr;
 
@@ -2065,13 +2071,13 @@ done:
 		transmit_msg_retry(node, msg);
 		cec_msg_init(&msg, from, CEC_LOG_ADDR_TV);
 		cec_msg_standby(&msg);
-		for (int i = 0; i < 50; i++) {
+		for (int i = 0; i < 10; i++) {
 			ret = transmit_msg_retry(node, msg);
 			if (!ret)
 				break;
 			usleep(200000);
 		}
-		if (ret)
+		if (ret != ENONET)
 			printf("%s\n", strerror(ret));
 		else
 			printf("OK\n");
@@ -2686,7 +2692,7 @@ int main(int argc, char **argv)
 				}
 			}
 			list_devices();
-			break;
+			return 0;
 
 		case OptTestStandbyWakeupCycle: {
 			static constexpr const char *arg_names[] = {
