@@ -151,6 +151,7 @@ struct parse_event key_events[] = {
 	{"KEY_SCALE", 120},
 	{"KEY_KPCOMMA", 121},
 	{"KEY_HANGEUL", 122},
+	{"KEY_HANGUEL", 122},
 	{"KEY_HANJA", 123},
 	{"KEY_YEN", 124},
 	{"KEY_LEFTMETA", 125},
@@ -181,6 +182,8 @@ struct parse_event key_events[] = {
 	{"KEY_WWW", 150},
 	{"KEY_MSDOS", 151},
 	{"KEY_COFFEE", 152},
+	{"KEY_SCREENLOCK", 152},
+	{"KEY_ROTATE_DISPLAY", 153},
 	{"KEY_DIRECTION", 153},
 	{"KEY_CYCLEWINDOWS", 154},
 	{"KEY_MAIL", 155},
@@ -227,6 +230,7 @@ struct parse_event key_events[] = {
 	{"KEY_PAUSECD", 201},
 	{"KEY_PROG3", 202},
 	{"KEY_PROG4", 203},
+	{"KEY_ALL_APPLICATIONS", 204},
 	{"KEY_DASHBOARD", 204},
 	{"KEY_SUSPEND", 205},
 	{"KEY_CLOSE", 206},
@@ -267,8 +271,10 @@ struct parse_event key_events[] = {
 	{"KEY_VIDEO_NEXT", 241},
 	{"KEY_VIDEO_PREV", 242},
 	{"KEY_BRIGHTNESS_CYCLE", 243},
+	{"KEY_BRIGHTNESS_AUTO", 244},
 	{"KEY_BRIGHTNESS_ZERO", 244},
 	{"KEY_DISPLAY_OFF", 245},
+	{"KEY_WWAN", 246},
 	{"KEY_WIMAX", 246},
 	{"KEY_RFKILL", 247},
 	{"KEY_MICMUTE", 248},
@@ -307,11 +313,11 @@ struct parse_event key_events[] = {
 	{"BTN_BASE6", 0x12b},
 	{"BTN_DEAD", 0x12f},
 	{"BTN_GAMEPAD", 0x130},
-	{"BTN_A", 0x130},
-	{"BTN_B", 0x131},
+	{"BTN_SOUTH", 0x130},
+	{"BTN_EAST", 0x131},
 	{"BTN_C", 0x132},
-	{"BTN_X", 0x133},
-	{"BTN_Y", 0x134},
+	{"BTN_NORTH", 0x133},
+	{"BTN_WEST", 0x134},
 	{"BTN_Z", 0x135},
 	{"BTN_TL", 0x136},
 	{"BTN_TR", 0x137},
@@ -332,6 +338,7 @@ struct parse_event key_events[] = {
 	{"BTN_TOOL_MOUSE", 0x146},
 	{"BTN_TOOL_LENS", 0x147},
 	{"BTN_TOOL_QUINTTAP", 0x148},
+	{"BTN_STYLUS3", 0x149},
 	{"BTN_TOUCH", 0x14a},
 	{"BTN_STYLUS", 0x14b},
 	{"BTN_STYLUS2", 0x14c},
@@ -361,9 +368,11 @@ struct parse_event key_events[] = {
 	{"KEY_TITLE", 0x171},
 	{"KEY_SUBTITLE", 0x172},
 	{"KEY_ANGLE", 0x173},
+	{"KEY_FULL_SCREEN", 0x174},
 	{"KEY_ZOOM", 0x174},
 	{"KEY_MODE", 0x175},
 	{"KEY_KEYBOARD", 0x176},
+	{"KEY_ASPECT_RATIO", 0x177},
 	{"KEY_SCREEN", 0x177},
 	{"KEY_PC", 0x178},
 	{"KEY_TV", 0x179},
@@ -421,6 +430,7 @@ struct parse_event key_events[] = {
 	{"KEY_ADDRESSBOOK", 0x1ad},
 	{"KEY_MESSENGER", 0x1ae},
 	{"KEY_DISPLAYTOGGLE", 0x1af},
+	{"KEY_BRIGHTNESS_TOGGLE", 0x1af},
 	{"KEY_SPELLCHECK", 0x1b0},
 	{"KEY_LOGOFF", 0x1b1},
 	{"KEY_DOLLAR", 0x1b2},
@@ -432,6 +442,9 @@ struct parse_event key_events[] = {
 	{"KEY_10CHANNELSUP", 0x1b8},
 	{"KEY_10CHANNELSDOWN", 0x1b9},
 	{"KEY_IMAGES", 0x1ba},
+	{"KEY_NOTIFICATION_CENTER", 0x1bc},
+	{"KEY_PICKUP_PHONE", 0x1bd},
+	{"KEY_HANGUP_PHONE", 0x1be},
 	{"KEY_DEL_EOL", 0x1c0},
 	{"KEY_DEL_EOS", 0x1c1},
 	{"KEY_INS_LINE", 0x1c2},
@@ -457,6 +470,7 @@ struct parse_event key_events[] = {
 	{"KEY_FN_F", 0x1e2},
 	{"KEY_FN_S", 0x1e3},
 	{"KEY_FN_B", 0x1e4},
+	{"KEY_FN_RIGHT_SHIFT", 0x1e5},
 	{"KEY_BRL_DOT1", 0x1f1},
 	{"KEY_BRL_DOT2", 0x1f2},
 	{"KEY_BRL_DOT3", 0x1f3},
@@ -479,6 +493,10 @@ struct parse_event key_events[] = {
 	{"KEY_NUMERIC_9", 0x209},
 	{"KEY_NUMERIC_STAR", 0x20a},
 	{"KEY_NUMERIC_POUND", 0x20b},
+	{"KEY_NUMERIC_A", 0x20c},
+	{"KEY_NUMERIC_B", 0x20d},
+	{"KEY_NUMERIC_C", 0x20e},
+	{"KEY_NUMERIC_D", 0x20f},
 	{"KEY_CAMERA_FOCUS", 0x210},
 	{"KEY_WPS_BUTTON", 0x211},
 	{"KEY_TOUCHPAD_TOGGLE", 0x212},
@@ -490,6 +508,118 @@ struct parse_event key_events[] = {
 	{"KEY_CAMERA_DOWN", 0x218},
 	{"KEY_CAMERA_LEFT", 0x219},
 	{"KEY_CAMERA_RIGHT", 0x21a},
+	{"KEY_ATTENDANT_ON", 0x21b},
+	{"KEY_ATTENDANT_OFF", 0x21c},
+	{"KEY_ATTENDANT_TOGGLE", 0x21d},
+	{"KEY_LIGHTS_TOGGLE", 0x21e},
+	{"BTN_DPAD_UP", 0x220},
+	{"BTN_DPAD_DOWN", 0x221},
+	{"BTN_DPAD_LEFT", 0x222},
+	{"BTN_DPAD_RIGHT", 0x223},
+	{"KEY_ALS_TOGGLE", 0x230},
+	{"KEY_ROTATE_LOCK_TOGGLE", 0x231},
+	{"KEY_REFRESH_RATE_TOGGLE", 0x232},
+	{"KEY_BUTTONCONFIG", 0x240},
+	{"KEY_TASKMANAGER", 0x241},
+	{"KEY_JOURNAL", 0x242},
+	{"KEY_CONTROLPANEL", 0x243},
+	{"KEY_APPSELECT", 0x244},
+	{"KEY_SCREENSAVER", 0x245},
+	{"KEY_VOICECOMMAND", 0x246},
+	{"KEY_ASSISTANT", 0x247},
+	{"KEY_KBD_LAYOUT_NEXT", 0x248},
+	{"KEY_EMOJI_PICKER", 0x249},
+	{"KEY_DICTATE", 0x24a},
+	{"KEY_CAMERA_ACCESS_ENABLE", 0x24b},
+	{"KEY_CAMERA_ACCESS_DISABLE", 0x24c},
+	{"KEY_CAMERA_ACCESS_TOGGLE", 0x24d},
+	{"KEY_ACCESSIBILITY", 0x24e},
+	{"KEY_DO_NOT_DISTURB", 0x24f},
+	{"KEY_BRIGHTNESS_MIN", 0x250},
+	{"KEY_BRIGHTNESS_MAX", 0x251},
+	{"KEY_KBDINPUTASSIST_PREV", 0x260},
+	{"KEY_KBDINPUTASSIST_NEXT", 0x261},
+	{"KEY_KBDINPUTASSIST_PREVGROUP", 0x262},
+	{"KEY_KBDINPUTASSIST_NEXTGROUP", 0x263},
+	{"KEY_KBDINPUTASSIST_ACCEPT", 0x264},
+	{"KEY_KBDINPUTASSIST_CANCEL", 0x265},
+	{"KEY_RIGHT_UP", 0x266},
+	{"KEY_RIGHT_DOWN", 0x267},
+	{"KEY_LEFT_UP", 0x268},
+	{"KEY_LEFT_DOWN", 0x269},
+	{"KEY_ROOT_MENU", 0x26a},
+	{"KEY_MEDIA_TOP_MENU", 0x26b},
+	{"KEY_NUMERIC_11", 0x26c},
+	{"KEY_NUMERIC_12", 0x26d},
+	{"KEY_AUDIO_DESC", 0x26e},
+	{"KEY_3D_MODE", 0x26f},
+	{"KEY_NEXT_FAVORITE", 0x270},
+	{"KEY_STOP_RECORD", 0x271},
+	{"KEY_PAUSE_RECORD", 0x272},
+	{"KEY_VOD", 0x273},
+	{"KEY_UNMUTE", 0x274},
+	{"KEY_FASTREVERSE", 0x275},
+	{"KEY_SLOWREVERSE", 0x276},
+	{"KEY_DATA", 0x277},
+	{"KEY_ONSCREEN_KEYBOARD", 0x278},
+	{"KEY_PRIVACY_SCREEN_TOGGLE", 0x279},
+	{"KEY_SELECTIVE_SCREENSHOT", 0x27a},
+	{"KEY_NEXT_ELEMENT", 0x27b},
+	{"KEY_PREVIOUS_ELEMENT", 0x27c},
+	{"KEY_AUTOPILOT_ENGAGE_TOGGLE", 0x27d},
+	{"KEY_MARK_WAYPOINT", 0x27e},
+	{"KEY_SOS", 0x27f},
+	{"KEY_NAV_CHART", 0x280},
+	{"KEY_FISHING_CHART", 0x281},
+	{"KEY_SINGLE_RANGE_RADAR", 0x282},
+	{"KEY_DUAL_RANGE_RADAR", 0x283},
+	{"KEY_RADAR_OVERLAY", 0x284},
+	{"KEY_TRADITIONAL_SONAR", 0x285},
+	{"KEY_CLEARVU_SONAR", 0x286},
+	{"KEY_SIDEVU_SONAR", 0x287},
+	{"KEY_NAV_INFO", 0x288},
+	{"KEY_BRIGHTNESS_MENU", 0x289},
+	{"KEY_MACRO1", 0x290},
+	{"KEY_MACRO2", 0x291},
+	{"KEY_MACRO3", 0x292},
+	{"KEY_MACRO4", 0x293},
+	{"KEY_MACRO5", 0x294},
+	{"KEY_MACRO6", 0x295},
+	{"KEY_MACRO7", 0x296},
+	{"KEY_MACRO8", 0x297},
+	{"KEY_MACRO9", 0x298},
+	{"KEY_MACRO10", 0x299},
+	{"KEY_MACRO11", 0x29a},
+	{"KEY_MACRO12", 0x29b},
+	{"KEY_MACRO13", 0x29c},
+	{"KEY_MACRO14", 0x29d},
+	{"KEY_MACRO15", 0x29e},
+	{"KEY_MACRO16", 0x29f},
+	{"KEY_MACRO17", 0x2a0},
+	{"KEY_MACRO18", 0x2a1},
+	{"KEY_MACRO19", 0x2a2},
+	{"KEY_MACRO20", 0x2a3},
+	{"KEY_MACRO21", 0x2a4},
+	{"KEY_MACRO22", 0x2a5},
+	{"KEY_MACRO23", 0x2a6},
+	{"KEY_MACRO24", 0x2a7},
+	{"KEY_MACRO25", 0x2a8},
+	{"KEY_MACRO26", 0x2a9},
+	{"KEY_MACRO27", 0x2aa},
+	{"KEY_MACRO28", 0x2ab},
+	{"KEY_MACRO29", 0x2ac},
+	{"KEY_MACRO30", 0x2ad},
+	{"KEY_MACRO_RECORD_START", 0x2b0},
+	{"KEY_MACRO_RECORD_STOP", 0x2b1},
+	{"KEY_MACRO_PRESET_CYCLE", 0x2b2},
+	{"KEY_MACRO_PRESET1", 0x2b3},
+	{"KEY_MACRO_PRESET2", 0x2b4},
+	{"KEY_MACRO_PRESET3", 0x2b5},
+	{"KEY_KBD_LCD_MENU1", 0x2b8},
+	{"KEY_KBD_LCD_MENU2", 0x2b9},
+	{"KEY_KBD_LCD_MENU3", 0x2ba},
+	{"KEY_KBD_LCD_MENU4", 0x2bb},
+	{"KEY_KBD_LCD_MENU5", 0x2bc},
 	{"BTN_TRIGGER_HAPPY", 0x2c0},
 	{"BTN_TRIGGER_HAPPY1", 0x2c0},
 	{"BTN_TRIGGER_HAPPY2", 0x2c1},
@@ -531,7 +661,6 @@ struct parse_event key_events[] = {
 	{"BTN_TRIGGER_HAPPY38", 0x2e5},
 	{"BTN_TRIGGER_HAPPY39", 0x2e6},
 	{"BTN_TRIGGER_HAPPY40", 0x2e7},
-	{"KEY_MAX", 0x2ff},
 	{ NULL, 0}
 };
 struct parse_event rel_events[] = {
@@ -545,6 +674,9 @@ struct parse_event rel_events[] = {
 	{"REL_DIAL", 0x07},
 	{"REL_WHEEL", 0x08},
 	{"REL_MISC", 0x09},
+	{"REL_RESERVED", 0x0a},
+	{"REL_WHEEL_HI_RES", 0x0b},
+	{"REL_HWHEEL_HI_RES", 0x0c},
 	{"REL_MAX", 0x0f},
 	{ NULL, 0}
 };
@@ -574,7 +706,9 @@ struct parse_event abs_events[] = {
 	{"ABS_TILT_Y", 0x1b},
 	{"ABS_TOOL_WIDTH", 0x1c},
 	{"ABS_VOLUME", 0x20},
+	{"ABS_PROFILE", 0x21},
 	{"ABS_MISC", 0x28},
+	{"ABS_RESERVED", 0x2e},
 	{"ABS_MT_SLOT", 0x2f},
 	{"ABS_MT_TOUCH_MAJOR", 0x30},
 	{"ABS_MT_TOUCH_MINOR", 0x31},
