@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2006 Mauro Carvalho Chehab <mchehab@infradead.org>
+   Copyright (C) 2006 Mauro Carvalho Chehab <mchehab@kernel.org>
 
    The libv4l2util Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -100,6 +100,8 @@ static char *prt_caps(uint32_t caps)
 {
 	static char s[4096]="";
 
+	s[0] = '\0';
+
 	if (V4L2_CAP_VIDEO_CAPTURE & caps)
 		strcat (s,"CAPTURE ");
 	if (V4L2_CAP_VIDEO_CAPTURE_MPLANE & caps)
@@ -140,14 +142,30 @@ static char *prt_caps(uint32_t caps)
 		strcat (s,"RADIO ");
 	if (V4L2_CAP_READWRITE & caps)
 		strcat (s,"READWRITE ");
-	if (V4L2_CAP_ASYNCIO & caps)
-		strcat (s,"ASYNCIO ");
 	if (V4L2_CAP_STREAMING & caps)
 		strcat (s,"STREAMING ");
 	if (V4L2_CAP_EXT_PIX_FORMAT & caps)
 		strcat (s,"EXT_PIX_FORMAT ");
 	if (V4L2_CAP_DEVICE_CAPS & caps)
 		strcat (s,"DEVICE_CAPS ");
+	if(V4L2_CAP_VIDEO_OUTPUT_OVERLAY & caps)
+		strcat (s,"VIDEO_OUTPUT_OVERLAY ");
+	if(V4L2_CAP_HW_FREQ_SEEK & caps)
+		strcat (s,"HW_FREQ_SEEK ");
+	if(V4L2_CAP_VIDEO_M2M_MPLANE & caps)
+		strcat (s,"VIDEO_M2M_MPLANE ");
+	if(V4L2_CAP_VIDEO_M2M & caps)
+		strcat (s,"VIDEO_M2M ");
+	if(V4L2_CAP_SDR_OUTPUT & caps)
+		strcat (s,"SDR_OUTPUT ");
+	if(V4L2_CAP_META_CAPTURE & caps)
+		strcat (s,"META_CAPTURE ");
+	if(V4L2_CAP_META_OUTPUT & caps)
+		strcat (s,"META_OUTPUT ");
+	if(V4L2_CAP_TOUCH & caps)
+		strcat (s,"TOUCH ");
+	if(V4L2_CAP_IO_MC & caps)
+		strcat (s,"IO_MC ");
 
 	return s;
 }
@@ -156,13 +174,13 @@ static void prt_buf_info(char *name,struct v4l2_buffer *p)
 {
 	struct v4l2_timecode *tc=&p->timecode;
 
-	printf ("%s: %02ld:%02d:%02d.%08ld index=%d, type=%s, "
+	printf ("%s: %02llu:%02d:%02d.%08llu index=%d, type=%s, "
 		"bytesused=%d, flags=0x%08x, "
 		"field=%s, sequence=%d, memory=%s, offset=0x%08x, length=%d\n",
-		name, (p->timestamp.tv_sec/3600),
+		name, (__u64)(p->timestamp.tv_sec/3600),
 		(int)(p->timestamp.tv_sec/60)%60,
 		(int)(p->timestamp.tv_sec%60),
-		p->timestamp.tv_usec,
+		(__u64)p->timestamp.tv_usec,
 		p->index,
 		prt_names(p->type,v4l2_type_names),
 		p->bytesused,p->flags,
@@ -436,7 +454,8 @@ int v4l2_gettryset_fmt_cap (struct v4l2_driver *drv, enum v4l2_direction dir,
 			perror("VIDIOC_G_FMT failed\n");
 		}
 		return ret;
-	} else if (dir & (~(V4L2_TRY|V4L2_SET)) ) {
+	}
+	if (dir & (~(V4L2_TRY | V4L2_SET)) ) {
 		perror ("Invalid direction\n");
 		return EINVAL;
 	}
