@@ -973,6 +973,7 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 	struct node node2;
 	struct v4l2_capability vcap = {};
 	struct v4l2_subdev_capability subdevcap = {};
+	struct v4l2_subdev_client_capability subdevclientcap = {};
 	std::string driver;
 
 	tests_total = tests_ok = warnings = 0;
@@ -993,6 +994,9 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 		determine_codec_mask(node);
 	} else if (node.is_subdev()) {
 		doioctl(&node, VIDIOC_SUBDEV_QUERYCAP, &subdevcap);
+		subdevclientcap.capabilities = ~0ULL;
+		if (doioctl(&node, VIDIOC_SUBDEV_S_CLIENT_CAP, &subdevclientcap))
+			subdevclientcap.capabilities = 0ULL;
 	} else {
 		memset(&vcap, 0, sizeof(vcap));
 	}
@@ -1079,7 +1083,7 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 		}
 	} else if (node.is_subdev()) {
 		printf("Driver Info:\n");
-		v4l2_info_subdev_capability(subdevcap);
+		v4l2_info_subdev_capability(subdevcap, subdevclientcap);
 	}
 
 	__u32 ent_id = 0;
@@ -1268,15 +1272,17 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 
 			for (unsigned which = V4L2_SUBDEV_FORMAT_TRY;
 				which <= V4L2_SUBDEV_FORMAT_ACTIVE; which++) {
+				struct v4l2_subdev_routing &routing = sd_routing[which];
 
-				sd_routing[which].which = which;
-				sd_routing[which].routes = (__u64)sd_routes[which];
-				sd_routing[which].num_routes = NUM_ROUTES_MAX;
+				routing.which = which;
+				routing.routes = (uintptr_t)sd_routes[which];
+				routing.len_routes = NUM_ROUTES_MAX;
+				routing.num_routes = 0;
 
-				ret = doioctl(&node, VIDIOC_SUBDEV_G_ROUTING, &sd_routing[which]);
+				ret = doioctl(&node, VIDIOC_SUBDEV_G_ROUTING, &routing);
 				if (ret) {
 					fail("VIDIOC_SUBDEV_G_ROUTING: failed to get routing\n");
-					sd_routing[which].num_routes = 0;
+					routing.num_routes = 0;
 				}
 			}
 		}
@@ -1301,7 +1307,7 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 					routes = sd_routes[which];
 				} else {
 					dummy_routing.num_routes = 1;
-					dummy_routing.routes = (__u64)&dummy_routes;
+					dummy_routing.routes = (uintptr_t)&dummy_routes;
 					dummy_routes[0].source_pad = pad;
 					dummy_routes[0].source_stream = 0;
 					dummy_routes[0].sink_pad = pad;
@@ -1341,8 +1347,9 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 					       which ? "Active" : "Try",
 					       ok(testSubDevSelection(&node, which, pad, stream)));
 					if (which)
-						printf("\ttest VIDIOC_SUBDEV_G/S_FRAME_INTERVAL: %s\n",
-						       ok(testSubDevFrameInterval(&node, pad, stream)));
+						printf("\ttest %s VIDIOC_SUBDEV_G/S_FRAME_INTERVAL: %s\n",
+						       which ? "Active" : "Try",
+						       ok(testSubDevFrameInterval(&node, which, pad, stream)));
 				}
 			}
 
@@ -1365,6 +1372,13 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 				if (node.has_subdev_selection &&
 				    node.has_subdev_selection != node.has_subdev_fmt)
 					fail("VIDIOC_SUBDEV_G/S_SELECTION: fmt/selection mismatch\n");
+				if (node.has_ival_uses_which()) {
+					if (node.has_subdev_frame_interval && node.has_subdev_frame_interval < 3)
+						fail("VIDIOC_SUBDEV_G/S_FRAME_INTERVAL: try/active mismatch\n");
+					if (node.has_subdev_frame_interval &&
+					    node.has_subdev_frame_interval != node.has_subdev_fmt)
+						fail("VIDIOC_SUBDEV_G/S_FRAME_INTERVAL: fmt/frame_interval mismatch\n");
+				}
 			}
 			printf("\n");
 		}
@@ -1454,6 +1468,7 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 		printf("Buffer ioctls%s:\n", suffix);
 		printf("\ttest VIDIOC_REQBUFS/CREATE_BUFS/QUERYBUF: %s\n", ok(testReqBufs(&node)));
 		printf("\ttest CREATE_BUFS maximum buffers: %s\n", ok(testCreateBufsMax(&node)));
+		printf("\ttest VIDIOC_REMOVE_BUFS: %s\n", ok(testRemoveBufs(&node)));
 		// Reopen after each streaming test to reset the streaming state
 		// in case of any errors in the preceeding test.
 		node.reopen();
