@@ -18,26 +18,17 @@
     Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
-#include <getopt.h>
+#include <map>
+#include <set>
+
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
-#include <sys/ioctl.h>
-#include <sys/time.h>
-#include <math.h>
+
 #include "v4l2-compliance.h"
 
 int testRegister(struct node *node)
 {
 	struct v4l2_dbg_register reg;
-	struct v4l2_dbg_chip_info chip;
+	struct v4l2_dbg_chip_info chip = {};
 	int ret;
 	int uid = getuid();
 
@@ -49,7 +40,8 @@ int testRegister(struct node *node)
 		return ret;
 	// Not allowed to call VIDIOC_DBG_G_REGISTER unless root
 	fail_on_test(uid && ret != EPERM);
-	fail_on_test(uid == 0 && ret);
+	fail_on_test(uid == 0 && ret && ret != EINVAL);
+	fail_on_test(uid == 0 && !ret && reg.size == 0);
 	chip.match.type = V4L2_CHIP_MATCH_BRIDGE;
 	chip.match.addr = 0;
 	fail_on_test(doioctl(node, VIDIOC_DBG_G_CHIP_INFO, &chip));
@@ -65,5 +57,5 @@ int testRegister(struct node *node)
 
 int testLogStatus(struct node *node)
 {
-	return doioctl(node, VIDIOC_LOG_STATUS, NULL);
+	return doioctl(node, VIDIOC_LOG_STATUS, nullptr);
 }
