@@ -49,7 +49,7 @@ static int testEnumFrameIntervals(struct node *node, __u32 pixfmt,
 				  __u32 w, __u32 h, __u32 type)
 {
 	struct v4l2_frmivalenum frmival;
-	struct v4l2_frmival_stepwise *sw = &frmival.stepwise;
+	const struct v4l2_frmival_stepwise *sw = &frmival.stepwise;
 	bool found_stepwise = false;
 	unsigned f = 0;
 	int ret;
@@ -129,7 +129,7 @@ static int testEnumFrameIntervals(struct node *node, __u32 pixfmt,
 static int testEnumFrameSizes(struct node *node, __u32 pixfmt)
 {
 	struct v4l2_frmsizeenum frmsize;
-	struct v4l2_frmsize_stepwise *sw = &frmsize.stepwise;
+	const struct v4l2_frmsize_stepwise *sw = &frmsize.stepwise;
 	bool found_stepwise = false;
 	__u64 cookie;
 	unsigned f = 0;
@@ -281,12 +281,23 @@ static int testEnumFormatsType(struct node *node, unsigned type)
 				      V4L2_FMT_FLAG_ENC_CAP_FRAME_INTERVAL |
 				      V4L2_FMT_FLAG_CSC_COLORSPACE |
 				      V4L2_FMT_FLAG_CSC_YCBCR_ENC | V4L2_FMT_FLAG_CSC_HSV_ENC |
-				      V4L2_FMT_FLAG_CSC_QUANTIZATION | V4L2_FMT_FLAG_CSC_XFER_FUNC))
+				      V4L2_FMT_FLAG_CSC_QUANTIZATION | V4L2_FMT_FLAG_CSC_XFER_FUNC |
+				      V4L2_FMT_FLAG_META_LINE_BASED))
 			return fail("unknown flag %08x returned\n", fmtdesc.flags);
 		if (!(fmtdesc.flags & V4L2_FMT_FLAG_COMPRESSED))
 			fail_on_test(fmtdesc.flags & (V4L2_FMT_FLAG_CONTINUOUS_BYTESTREAM |
 						      V4L2_FMT_FLAG_DYN_RESOLUTION |
 						      V4L2_FMT_FLAG_ENC_CAP_FRAME_INTERVAL));
+
+		// Checks for metadata formats.
+		// The META_LINE_BASED flag can be set for metadata formats only.
+		if (type == V4L2_BUF_TYPE_META_OUTPUT || type == V4L2_BUF_TYPE_META_CAPTURE)
+			fail_on_test(fmtdesc.flags & ~V4L2_FMT_FLAG_META_LINE_BASED);
+		// Only the META_LINE_BASED flag is valid for metadata formats.
+		if (fmtdesc.flags & V4L2_FMT_FLAG_META_LINE_BASED)
+			fail_on_test(type != V4L2_BUF_TYPE_META_OUTPUT &&
+				     type != V4L2_BUF_TYPE_META_CAPTURE);
+
 		ret = testEnumFrameSizes(node, fmtdesc.pixelformat);
 		if (ret)
 			fail_on_test(node->codec_mask & STATEFUL_ENCODER);
@@ -438,15 +449,15 @@ static void createInvalidFmt(struct v4l2_format &fmt, struct v4l2_clip &clip, un
 
 static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4l2_format &fmt, bool have_clip = false)
 {
-	pixfmt_map &map = node->buftype_pixfmts[type];
-	pixfmt_map *map_splane;
-	struct v4l2_pix_format &pix = fmt.fmt.pix;
-	struct v4l2_pix_format_mplane &pix_mp = fmt.fmt.pix_mp;
-	struct v4l2_window &win = fmt.fmt.win;
-	struct v4l2_vbi_format &vbi = fmt.fmt.vbi;
-	struct v4l2_sliced_vbi_format &sliced = fmt.fmt.sliced;
-	struct v4l2_sdr_format &sdr = fmt.fmt.sdr;
-	struct v4l2_meta_format &meta = fmt.fmt.meta;
+	const pixfmt_map &map = node->buftype_pixfmts[type];
+	const pixfmt_map *map_splane;
+	const struct v4l2_pix_format &pix = fmt.fmt.pix;
+	const struct v4l2_pix_format_mplane &pix_mp = fmt.fmt.pix_mp;
+	const struct v4l2_window &win = fmt.fmt.win;
+	const struct v4l2_vbi_format &vbi = fmt.fmt.vbi;
+	const struct v4l2_sliced_vbi_format &sliced = fmt.fmt.sliced;
+	const struct v4l2_sdr_format &sdr = fmt.fmt.sdr;
+	const struct v4l2_meta_format &meta = fmt.fmt.meta;
 	unsigned min_data_samples;
 	unsigned min_sampling_rate;
 	v4l2_std_id std;
@@ -497,7 +508,7 @@ static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4
 			return fail("pix_mp.reserved not zeroed\n");
 		fail_on_test(pix_mp.num_planes == 0 || pix_mp.num_planes >= VIDEO_MAX_PLANES);
 		for (int i = 0; i < pix_mp.num_planes; i++) {
-			struct v4l2_plane_pix_format &pfmt = pix_mp.plane_fmt[i];
+			const struct v4l2_plane_pix_format &pfmt = pix_mp.plane_fmt[i];
 
 			ret = check_0(pfmt.reserved, sizeof(pfmt.reserved));
 			if (ret)
@@ -559,7 +570,7 @@ static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4
 		if (have_clip)
 			fail_on_test(!win.clipcount && (node->fbuf_caps & V4L2_FBUF_CAP_LIST_CLIPPING));
 		if (win.clipcount) {
-			struct v4l2_rect *r = &win.clips->c;
+			const struct v4l2_rect *r = &win.clips->c;
 			struct v4l2_framebuffer fb;
 
 			fail_on_test(doioctl(node, VIDIOC_G_FBUF, &fb));
@@ -590,6 +601,10 @@ static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4
 			return fail("dataformat %08x (%s) for buftype %d not reported by ENUM_FMT\n",
 					meta.dataformat, fcc2s(meta.dataformat).c_str(), type);
 		fail_on_test(meta.buffersize == 0);
+		if (map.at(meta.dataformat) & V4L2_FMT_FLAG_META_LINE_BASED) {
+			fail_on_test(!meta.width || !meta.height);
+			fail_on_test(!meta.bytesperline);
+		}
 		break;
 	case V4L2_BUF_TYPE_PRIVATE:
 		break;
@@ -660,6 +675,8 @@ static bool matchFormats(const struct v4l2_format &f1, const struct v4l2_format
 {
 	const struct v4l2_pix_format &pix1 = f1.fmt.pix;
 	const struct v4l2_pix_format &pix2 = f2.fmt.pix;
+	const struct v4l2_pix_format_mplane &pix_mp1 = f1.fmt.pix_mp;
+	const struct v4l2_pix_format_mplane &pix_mp2 = f2.fmt.pix_mp;
 	const struct v4l2_window &win1 = f1.fmt.win;
 	const struct v4l2_window &win2 = f2.fmt.win;
 
@@ -696,7 +713,19 @@ static bool matchFormats(const struct v4l2_format &f1, const struct v4l2_format
 		return !memcmp(&f1.fmt.sliced, &f2.fmt.sliced, sizeof(f1.fmt.sliced));
 	case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
 	case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
-		return !memcmp(&f1.fmt.pix_mp, &f2.fmt.pix_mp, sizeof(f1.fmt.pix_mp));
+		if (!memcmp(&f1.fmt.pix_mp, &f2.fmt.pix_mp, sizeof(f1.fmt.pix_mp)))
+			return true;
+		printf("\t\tG_FMT:     %dx%d, %s, %d, %d, %d, %d, %d\n",
+			pix_mp1.width, pix_mp1.height, fcc2s(pix_mp1.pixelformat).c_str(), pix_mp1.num_planes,
+			pix_mp1.field, pix_mp1.colorspace, pix_mp1.ycbcr_enc, pix_mp1.quantization);
+		for (unsigned p = 0; p < pix_mp1.num_planes; p++)
+			printf("\t\t\t%d: %d, %d\n", p, pix_mp1.plane_fmt[p].sizeimage, pix_mp1.plane_fmt[p].bytesperline);
+		printf("\t\tTRY/S_FMT: %dx%d, %s, %d, %d, %d, %d, %d\n",
+			pix_mp2.width, pix_mp2.height, fcc2s(pix_mp2.pixelformat).c_str(), pix_mp2.num_planes,
+			pix_mp2.field, pix_mp2.colorspace, pix_mp2.ycbcr_enc, pix_mp2.quantization);
+		for (unsigned p = 0; p < pix_mp2.num_planes; p++)
+			printf("\t\t\t%d: %d, %d\n", p, pix_mp2.plane_fmt[p].sizeimage, pix_mp2.plane_fmt[p].bytesperline);
+		return false;
 	case V4L2_BUF_TYPE_SDR_CAPTURE:
 	case V4L2_BUF_TYPE_SDR_OUTPUT:
 		return !memcmp(&f1.fmt.sdr, &f2.fmt.sdr, sizeof(f1.fmt.sdr));
@@ -1281,8 +1310,8 @@ int testSlicedVBICap(struct node *node)
 static int testParmStruct(struct node *node, struct v4l2_streamparm &parm)
 {
 	bool is_stateful_enc = node->codec_mask & STATEFUL_ENCODER;
-	struct v4l2_captureparm *cap = &parm.parm.capture;
-	struct v4l2_outputparm *out = &parm.parm.output;
+	const struct v4l2_captureparm *cap = &parm.parm.capture;
+	const struct v4l2_outputparm *out = &parm.parm.output;
 	int ret;
 
 	switch (parm.type) {
