@@ -1,31 +1,14 @@
-#include <cstdlib>
 #include <cstring>
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <inttypes.h>
-#include <getopt.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
 #include <netdb.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
-#include <sys/ioctl.h>
-#include <sys/time.h>
-#include <sys/mman.h>
-#include <dirent.h>
-#include <math.h>
+#include <sys/types.h>
+
 #include <linux/media.h>
 
 #include "compiler.h"
 #include "v4l2-ctl.h"
 #include "v4l-stream.h"
 #include <media-info.h>
-#include <fwht-ctrls.h>
 
 extern "C" {
 #include "v4l2-tpg.h"
@@ -35,7 +18,9 @@ static unsigned stream_count;
 static unsigned stream_skip;
 static __u32 memory = V4L2_MEMORY_MMAP;
 static __u32 out_memory = V4L2_MEMORY_MMAP;
-static int stream_sleep = -1;
+static int stream_sleep_count = -1;
+static int stream_sleep_ms = 1000;
+static unsigned stream_sleep_mode = 1;
 static bool stream_no_query;
 static unsigned stream_pat;
 static bool stream_loop;
@@ -106,8 +91,11 @@ enum codec_type {
 	DECODER
 };
 
+static enum codec_type codec_type;
+
 #define QUEUE_ERROR -1
 #define QUEUE_STOPPED -2
+#define QUEUE_OFF_ON -3
 
 class fps_timestamps {
 private:
@@ -145,6 +133,50 @@ public:
 	unsigned dropped();
 };
 
+static bool need_sleep(unsigned count)
+{
+	if (stream_sleep_count <= 0)
+		return false;
+
+	if (!(stream_sleep_mode & 1) && count > (unsigned)stream_sleep_count)
+		return false;
+
+	return !(count % stream_sleep_count);
+}
+
+static void do_sleep()
+{
+	int ms = stream_sleep_ms;
+
+	if (!ms) {
+		// For modes 2 and 3 just don't sleep in this case
+		if (stream_sleep_mode >= 2) {
+			stderr_info("streamoff, streamon\n");
+			return;
+		}
+
+		stderr_info("sleeping forever...\n");
+		while (1)
+			sleep(100);
+	}
+
+	if (ms < 0)
+		ms = ((__u64)-ms * rand()) / RAND_MAX + 1;
+
+	if (stream_sleep_mode >= 2)
+		stderr_info("streamoff, sleep %d ms, streamon\n", ms);
+	else if (stream_sleep_ms < 0 || verbose)
+		stderr_info("sleep %d ms\n", ms);
+	else
+		stderr_info("\n");
+	fflush(stderr);
+
+	struct timespec t;
+	t.tv_sec = ms / 1000;
+	t.tv_nsec = (ms % 1000) * 1000000;
+	nanosleep(&t, NULL);
+}
+
 void fps_timestamps::determine_field(int fd, unsigned type)
 {
 	struct v4l2_format fmt = { };
@@ -269,10 +301,18 @@ void streaming_usage()
 	       "                     skipped buffers as is passed by --stream-skip.\n"
 	       "  --stream-skip <count>\n"
 	       "                     skip the first <count> buffers. The default is 0.\n"
-	       "  --stream-sleep <count>\n"
-	       "                     sleep for 1 second every <count> buffers. If <count> is 0,\n"
-	       "                     then sleep forever right after streaming starts. The default\n"
-	       "                     is -1 (never sleep).\n"
+	       "  --stream-sleep count=<c>,sleep=<ms>,mode=<mode>\n"
+	       "                     Sleep for <ms> milliseconds (default=1000) after <c> buffers.\n"
+	       "                     If <c> is 0, then only sleep right after streaming starts.\n"
+	       "                     If <ms> is 0, then sleep forever (modes 0 and 1) or not at all\n"
+	       "                     (for modes 2 and 3), if <ms> is positive, then sleep for <ms>\n"
+	       "                     milliseconds, if <ms> is negative, then sleep for a random\n"
+	       "                     time between 1 and -<ms> milliseconds.\n"
+	       "                     There are different modes for this:\n"
+	       "                     <mode>=0: the sleep happens only once after <c> buffers.\n"
+	       "                     <mode>=1: the sleep happens every <c> buffers (default).\n"
+	       "                     <mode>=2: as 0, but call STREAMOFF/ON before/after the sleep.\n"
+	       "                     <mode>=3: as 1, but call STREAMOFF/ON before/after the sleep.\n"
 #ifndef NO_STREAM_TO
 	       "  --stream-to <file> stream to this file. The default is to discard the\n"
 	       "                     data. If <file> is '-', then the data is written to stdout\n"
@@ -285,6 +325,10 @@ void streaming_usage()
 #endif
 	       "  --stream-poll      use non-blocking mode and select() to stream.\n"
 	       "  --stream-buf-caps  show capture buffer capabilities\n"
+	       "  --stream-show-delta-now\n"
+	       "                     output the difference between the buffer timestamp and current\n"
+	       "                     clock, if the buffer timestamp source is the monotonic clock.\n"
+	       "                     Requires --verbose as well.\n"
 	       "  --stream-mmap <count>\n"
 	       "                     capture video using mmap() [VIDIOC_(D)QBUF]\n"
 	       "                     count: the number of buffers to allocate. The default is 3.\n"
@@ -311,10 +355,10 @@ void streaming_usage()
 	       "                     show a border around the pillar/letterboxed video.\n"
 	       "  --stream-out-sav   insert an SAV code in every line.\n"
 	       "  --stream-out-eav   insert an EAV code in every line.\n"
-	       "  --stream-out-pixel-aspect <aspect\n"
+	       "  --stream-out-pixel-aspect <aspect>\n"
 	       "                     select a pixel aspect ratio. The default is to autodetect.\n"
 	       "                     <aspect> can be one of: square, ntsc, pal\n"
-	       "  --stream-out-video-aspect <aspect\n"
+	       "  --stream-out-video-aspect <aspect>\n"
 	       "                     select a video aspect ratio. The default is to use the frame ratio.\n"
 	       "                     <aspect> can be one of: 4x3, 14x9, 16x9, anamorphic\n"
 	       "  --stream-out-alpha <alpha-value>\n"
@@ -343,7 +387,7 @@ void streaming_usage()
 	       "  --stream-out-dmabuf\n"
 	       "                     output video using dmabuf [VIDIOC_(D)QBUF]\n"
 	       "                     Requires a corresponding --stream-mmap option.\n"
-	       "  --list-patterns    list available patterns for use with --stream-pattern.\n"
+	       "  --list-patterns    list available patterns for use with --stream-out-pattern.\n"
 	       "  --list-buffers     list all video buffers [VIDIOC_QUERYBUF]\n"
 	       "  --list-buffers-out list all video output buffers [VIDIOC_QUERYBUF]\n"
 	       "  --list-buffers-vbi list all VBI buffers [VIDIOC_QUERYBUF]\n"
@@ -365,7 +409,7 @@ void streaming_usage()
 	       	V4L_STREAM_PORT);
 }
 
-static enum codec_type get_codec_type(cv4l_fd &fd)
+static void get_codec_type(cv4l_fd &fd)
 {
 	cv4l_disable_trace dt(fd);
 	struct v4l2_fmtdesc fmt_desc = {};
@@ -374,11 +418,13 @@ static enum codec_type get_codec_type(cv4l_fd &fd)
 	int num_out_fmts = 0;
 	int num_compressed_out_fmts = 0;
 
+	codec_type = NOT_CODEC;
+
 	if (!fd.has_vid_m2m())
-		return NOT_CODEC;
+		return;
 
 	if (fd.enum_fmt(fmt_desc, true, 0, fd.g_type()))
-		return NOT_CODEC;
+		return;
 
 	do {
 		if (fmt_desc.flags & V4L2_FMT_FLAG_COMPRESSED)
@@ -388,7 +434,7 @@ static enum codec_type get_codec_type(cv4l_fd &fd)
 
 
 	if (fd.enum_fmt(fmt_desc, true, 0, v4l_type_invert(fd.g_type())))
-		return NOT_CODEC;
+		return;
 
 	do {
 		if (fmt_desc.flags & V4L2_FMT_FLAG_COMPRESSED)
@@ -397,17 +443,17 @@ static enum codec_type get_codec_type(cv4l_fd &fd)
 	} while (!fd.enum_fmt(fmt_desc));
 
 	if (num_compressed_out_fmts == 0 && num_compressed_cap_fmts == num_cap_fmts) {
-		return ENCODER;
+		codec_type = ENCODER;
+		return;
 	}
 
 	if (num_compressed_cap_fmts == 0 && num_compressed_out_fmts == num_out_fmts) {
-		return DECODER;
+		codec_type = DECODER;
+		return;
 	}
-
-	return NOT_CODEC;
 }
 
-static int get_cap_compose_rect(cv4l_fd &fd)
+static void get_cap_compose_rect(cv4l_fd &fd)
 {
 	cv4l_disable_trace dt(fd);
 	v4l2_selection sel;
@@ -420,14 +466,12 @@ static int get_cap_compose_rect(cv4l_fd &fd)
 		support_cap_compose = true;
 		composed_width = sel.r.width;
 		composed_height = sel.r.height;
-		return 0;
-	}
-
-	support_cap_compose = false;
-	return 0;
+	} else {
+		support_cap_compose = false;
+    }
 }
 
-static int get_out_crop_rect(cv4l_fd &fd)
+static void get_out_crop_rect(cv4l_fd &fd)
 {
 	cv4l_disable_trace dt(fd);
 	v4l2_selection sel;
@@ -440,11 +484,9 @@ static int get_out_crop_rect(cv4l_fd &fd)
 		support_out_crop = true;
 		cropped_width = sel.r.width;
 		cropped_height = sel.r.height;
-		return 0;
+	} else {
+		support_out_crop = false;
 	}
-
-	support_out_crop = false;
-	return 0;
 }
 
 static void set_time_stamp(cv4l_buffer &buf)
@@ -516,7 +558,7 @@ static void print_buffer(FILE *f, struct v4l2_buffer &buf)
 		static_cast<__u64>(buf.timestamp.tv_sec), static_cast<__u64>(buf.timestamp.tv_usec),
 		timestamp_type2s(buf.flags).c_str(), timestamp_src2s(buf.flags).c_str());
 	if (buf.flags & V4L2_BUF_FLAG_TIMECODE) {
-		static const int fps_types[] = { 0, 24, 25, 30, 50, 60 };
+		static constexpr int fps_types[] = { 0, 24, 25, 30, 50, 60 };
 		int fps = buf.timecode.type;
 
 		if (fps > 5)
@@ -572,8 +614,17 @@ static void print_concise_buffer(FILE *f, cv4l_buffer &buf, cv4l_fmt &fmt,
 	if (!skip_ts && (buf.g_flags() & V4L2_BUF_FLAG_TIMESTAMP_MASK) != V4L2_BUF_FLAG_TIMESTAMP_COPY) {
 		double ts = buf.g_timestamp().tv_sec + buf.g_timestamp().tv_usec / 1000000.0;
 		fprintf(f, " ts: %.06f", ts);
-		if (last_ts <= 0.0)
+		if (last_ts != 0.0)
 			fprintf(f, " delta: %.03f ms", (ts - last_ts) * 1000.0);
+		if (options[OptStreamShowDeltaNow] &&
+		    (buf.g_flags() & V4L2_BUF_FLAG_TIMESTAMP_MASK) ==
+		    V4L2_BUF_FLAG_TIMESTAMP_MONOTONIC) {
+			timespec ts_clock;
+
+			clock_gettime(CLOCK_MONOTONIC, &ts_clock);
+			fprintf(f, " delta now: %+.03f ms",
+				((ts_clock.tv_sec + ts_clock.tv_nsec / 1000000000.0) - ts) * 1000.0);
+		}
 		last_ts = ts;
 
 		if (fps_ts.has_fps(true))
@@ -585,6 +636,8 @@ static void print_concise_buffer(FILE *f, cv4l_buffer &buf, cv4l_fmt &fmt,
 			fprintf(stderr, " dropped: %u", dropped);
 	}
 
+	fprintf(f, " field: %s", field2s(buf.g_field()).c_str());
+
 	__u32 fl = buf.g_flags() & (V4L2_BUF_FLAG_ERROR |
 				    V4L2_BUF_FLAG_KEYFRAME |
 				    V4L2_BUF_FLAG_PFRAME |
@@ -614,9 +667,16 @@ static void stream_buf_caps(cv4l_fd &fd, unsigned buftype)
 	cbufs.format.type = buftype;
 	cbufs.memory = V4L2_MEMORY_MMAP;
 	if (!v4l_ioctl(fd.g_v4l_fd(), VIDIOC_CREATE_BUFS, &cbufs)) {
-		printf("Streaming I/O Capabilities for %s: %s\n",
+		bool has_max_num_buffers =
+			cbufs.capabilities & V4L2_BUF_CAP_SUPPORTS_MAX_NUM_BUFFERS;
+
+		cbufs.capabilities &= ~V4L2_BUF_CAP_SUPPORTS_MAX_NUM_BUFFERS;
+		printf("Streaming I/O Capabilities for %s: %s",
 		       buftype2s(buftype).c_str(),
 		       bufcap2s(cbufs.capabilities).c_str());
+		if (has_max_num_buffers)
+			printf(", max-num-buffers=%u", cbufs.max_num_buffers);
+		printf("\n");
 		return;
 	}
 	v4l2_requestbuffers rbufs;
@@ -648,18 +708,44 @@ static void list_buffers(cv4l_fd &fd, unsigned buftype)
 
 void streaming_cmd(int ch, char *optarg)
 {
+	char *value, *subs;
 	unsigned i;
 	int speed;
 
 	switch (ch) {
 	case OptStreamCount:
-		stream_count = strtoul(optarg, 0L, 0);
+		stream_count = strtoul(optarg, nullptr, 0);
 		break;
 	case OptStreamSkip:
-		stream_skip = strtoul(optarg, 0L, 0);
+		stream_skip = strtoul(optarg, nullptr, 0);
 		break;
 	case OptStreamSleep:
-		stream_sleep = strtol(optarg, 0L, 0);
+		subs = optarg;
+		while (*subs != '\0') {
+			static constexpr const char *subopts[] = {
+				"count",
+				"sleep",
+				"mode",
+				nullptr
+			};
+
+			switch (parse_subopt(&subs, subopts, &value)) {
+			case 0:
+				stream_sleep_count = strtoul(value, nullptr, 0);
+				break;
+			case 1:
+				stream_sleep_ms = strtoul(value, nullptr, 0);
+				break;
+			case 2:
+				stream_sleep_mode = strtoul(value, nullptr, 0);
+				if (stream_sleep_mode < 4)
+					break;
+				fallthrough;
+			default:
+				streaming_usage();
+				std::exit(EXIT_FAILURE);
+			}
+		}
 		break;
 	case OptStreamNoQuery:
 		stream_no_query = true;
@@ -668,7 +754,7 @@ void streaming_cmd(int ch, char *optarg)
 		stream_loop = true;
 		break;
 	case OptStreamOutPattern:
-		stream_pat = strtoul(optarg, 0L, 0);
+		stream_pat = strtoul(optarg, nullptr, 0);
 		for (i = 0; tpg_pattern_strings[i]; i++) ;
 		if (stream_pat >= i)
 			stream_pat = 0;
@@ -708,7 +794,7 @@ void streaming_cmd(int ch, char *optarg)
 			streaming_usage();
 		break;
 	case OptStreamOutAlphaComponent:
-		stream_out_alpha = strtoul(optarg, 0L, 0);
+		stream_out_alpha = strtoul(optarg, nullptr, 0);
 		break;
 	case OptStreamOutAlphaRedOnly:
 		stream_out_alpha_red_only = true;
@@ -718,7 +804,7 @@ void streaming_cmd(int ch, char *optarg)
 		break;
 	case OptStreamOutHorSpeed:
 	case OptStreamOutVertSpeed:
-		speed = strtol(optarg, 0L, 0);
+		speed = strtol(optarg, nullptr, 0);
 		if (speed < -3)
 			speed = -3;
 		if (speed > 3)
@@ -729,7 +815,7 @@ void streaming_cmd(int ch, char *optarg)
 			stream_out_vert_mode = static_cast<tpg_move_mode>(speed + 3);
 		break;
 	case OptStreamOutPercFill:
-		stream_out_perc_fill = strtoul(optarg, 0L, 0);
+		stream_out_perc_fill = strtoul(optarg, nullptr, 0);
 		if (stream_out_perc_fill > 100)
 			stream_out_perc_fill = 100;
 		if (stream_out_perc_fill < 1)
@@ -769,7 +855,7 @@ void streaming_cmd(int ch, char *optarg)
 		fallthrough;
 	case OptStreamMmap:
 		if (optarg) {
-			reqbufs_count_cap = strtoul(optarg, 0L, 0);
+			reqbufs_count_cap = strtoul(optarg, nullptr, 0);
 			if (reqbufs_count_cap == 0)
 				reqbufs_count_cap = 3;
 		}
@@ -782,7 +868,7 @@ void streaming_cmd(int ch, char *optarg)
 		fallthrough;
 	case OptStreamOutMmap:
 		if (optarg) {
-			reqbufs_count_out = strtoul(optarg, 0L, 0);
+			reqbufs_count_out = strtoul(optarg, nullptr, 0);
 			if (reqbufs_count_out == 0)
 				reqbufs_count_out = 3;
 		}
@@ -835,7 +921,7 @@ static void set_fwht_stateless_params(struct v4l2_ctrl_fwht_params &fwht_params,
 	 * it is better not to reference it so the error won't propagate
 	 */
 	if (!last_bf_ts)
-		fwht_params.flags |= FWHT_FL_I_FRAME;
+		fwht_params.flags |= V4L2_FWHT_FL_I_FRAME;
 }
 
 static int alloc_fwht_req(int media_fd, unsigned index)
@@ -879,10 +965,10 @@ static bool set_fwht_req_by_fd(const struct fwht_cframe_hdr *hdr,
 
 	set_fwht_stateless_params(fwht_params, hdr, last_bf_ts);
 
-	for (int idx = 0; idx < VIDEO_MAX_FRAME; idx++) {
-		if (fwht_reqs[idx].fd == req_fd) {
-			fwht_reqs[idx].ts = ts;
-			fwht_reqs[idx].params = fwht_params;
+	for (auto &fwht_req : fwht_reqs) {
+		if (fwht_req.fd == req_fd) {
+			fwht_req.ts = ts;
+			fwht_req.params = fwht_params;
 			return true;
 		}
 	}
@@ -901,7 +987,7 @@ static int set_fwht_ext_ctrl(cv4l_fd &fd, const struct fwht_cframe_hdr *hdr,
 
 	set_fwht_stateless_params(fwht_params, hdr, last_bf_ts);
 
-	control.id = V4L2_CID_MPEG_VIDEO_FWHT_PARAMS;
+	control.id = V4L2_CID_STATELESS_FWHT_PARAMS;
 	control.ptr = &fwht_params;
 	control.size = sizeof(fwht_params);
 	controls.which = V4L2_CTRL_WHICH_REQUEST_VAL;
@@ -1113,7 +1199,8 @@ restart:
 		if (fmt.g_pixelformat() == V4L2_PIX_FMT_FWHT_STATELESS)
 			res = read_fwht_frame(fmt, static_cast<unsigned char *>(buf), fin,
 					      sz, expected_len, buf_len);
-		else if (support_out_crop && v4l2_fwht_find_pixfmt(fmt.g_pixelformat()))
+		else if (codec_type != NOT_CODEC && support_out_crop &&
+			 v4l2_fwht_find_pixfmt(fmt.g_pixelformat()))
 			res = read_write_padded_frame(fmt, static_cast<unsigned char *>(buf),
 						      fin, sz, expected_len, buf_len, true);
 		else
@@ -1157,8 +1244,10 @@ static int do_setup_out_buffers(cv4l_fd &fd, cv4l_queue &q, FILE *fin, bool qbuf
 			q.g_type() == V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE;
 	bool is_meta = q.g_type() == V4L2_BUF_TYPE_META_OUTPUT;
 
-	if (q.obtain_bufs(&fd))
+	if (q.obtain_bufs(&fd)) {
+		fprintf(stderr, "%s q.obtain_bufs failed\n", __func__);
 		return QUEUE_ERROR;
+	}
 
 	fd.g_fmt(fmt, q.g_type());
 	{
@@ -1230,8 +1319,10 @@ static int do_setup_out_buffers(cv4l_fd &fd, cv4l_queue &q, FILE *fin, bool qbuf
 	for (unsigned i = 0; i < q.g_buffers(); i++) {
 		cv4l_buffer buf(q);
 
-		if (fd.querybuf(buf, i))
+		if (fd.querybuf(buf, i)) {
+			fprintf(stderr, "%s fd.querybuf failed\n", __func__);
 			return QUEUE_ERROR;
+		}
 
 		buf.update(q, i);
 		for (unsigned j = 0; j < q.g_num_planes(); j++)
@@ -1258,7 +1349,9 @@ static int do_setup_out_buffers(cv4l_fd &fd, cv4l_queue &q, FILE *fin, bool qbuf
 			return QUEUE_STOPPED;
 
 		if (fmt.g_pixelformat() == V4L2_PIX_FMT_FWHT_STATELESS) {
-			int media_fd = mi_get_media_fd(fd.g_fd());
+			struct v4l2_capability vcap = {};
+			fd.querycap(vcap);
+			int media_fd = mi_get_media_fd(fd.g_fd(), (const char *)vcap.bus_info);
 
 			if (media_fd < 0) {
 				fprintf(stderr, "%s: mi_get_media_fd failed\n", __func__);
@@ -1285,7 +1378,7 @@ static int do_setup_out_buffers(cv4l_fd &fd, cv4l_queue &q, FILE *fin, bool qbuf
 				return QUEUE_ERROR;
 			tpg_update_mv_count(&tpg, V4L2_FIELD_HAS_T_OR_B(field));
 			if (!verbose)
-				fprintf(stderr, ">");
+				stderr_info(">");
 			fflush(stderr);
 			if (!ignore_count_skip && stream_count)
 				if (!--stream_count)
@@ -1367,7 +1460,8 @@ static void write_buffer_to_file(cv4l_fd &fd, cv4l_queue &q, cv4l_buffer &buf,
 		}
 		if (host_fd_to >= 0)
 			sz = fwrite(comp_ptr[j] + offset, 1, used, fout);
-		else if (support_cap_compose && v4l2_fwht_find_pixfmt(fmt.g_pixelformat()))
+		else if (codec_type != NOT_CODEC && support_cap_compose &&
+			 v4l2_fwht_find_pixfmt(fmt.g_pixelformat()))
 			read_write_padded_frame(fmt, static_cast<u8 *>(q.g_dataptr(buf.g_index(), j)) + offset,
 						fout, sz, used, used, false);
 		else
@@ -1432,7 +1526,7 @@ static int do_handle_cap(cv4l_fd &fd, cv4l_queue &q, FILE *fout, int *index,
 				     host_fd_to >= 0 ? 100 - comp_perc / comp_perc_count : -1);
 		comp_perc_count = comp_perc = 0;
 	}
-	if (!last_buffer && index == NULL) {
+	if (!last_buffer && index == nullptr) {
 		/*
 		 * EINVAL in qbuf can happen if this is the last buffer before
 		 * a dynamic resolution change sequence. In this case the buffer
@@ -1448,19 +1542,19 @@ static int do_handle_cap(cv4l_fd &fd, cv4l_queue &q, FILE *fout, int *index,
 		*index = buf.g_index();
 
 	if (!verbose) {
-		fprintf(stderr, "%c", ch);
+		stderr_info("%c", ch);
 		fflush(stderr);
 
 		if (fps_ts.has_fps()) {
 			unsigned dropped = fps_ts.dropped();
 
-			fprintf(stderr, " %.02f fps", fps_ts.fps());
+			stderr_info(" %.02f fps", fps_ts.fps());
 			if (dropped)
-				fprintf(stderr, ", dropped buffers: %u", dropped);
+				stderr_info(", dropped buffers: %u", dropped);
 			if (host_fd_to >= 0)
-				fprintf(stderr, " %d%% compression", 100 - comp_perc / comp_perc_count);
+				stderr_info(" %d%% compression", 100 - comp_perc / comp_perc_count);
 			comp_perc_count = comp_perc = 0;
-			fprintf(stderr, "\n");
+			stderr_info("\n");
 		}
 	}
 	count++;
@@ -1468,8 +1562,11 @@ static int do_handle_cap(cv4l_fd &fd, cv4l_queue &q, FILE *fout, int *index,
 	if (ignore_count_skip)
 		return 0;
 
-	if (stream_sleep > 0 && count % stream_sleep == 0)
-		sleep(1);
+	if (need_sleep(count)) {
+		if (stream_sleep_mode & 2)
+			return QUEUE_OFF_ON;
+		do_sleep();
+	}
 
 	if (is_empty_frame || is_error_frame)
 		return 0;
@@ -1499,6 +1596,11 @@ static int do_handle_out(cv4l_fd &fd, cv4l_queue &q, FILE *fin, cv4l_buffer *cap
 	if (cap) {
 		buf.s_index(cap->g_index());
 
+		if (fd.querybuf(buf)) {
+			fprintf(stderr, "%s fd.querybuf failed\n", __func__);
+			return QUEUE_ERROR;
+		}
+
 		for (unsigned j = 0; j < buf.g_num_planes(); j++) {
 			unsigned data_offset = cap->g_data_offset(j);
 
@@ -1507,8 +1609,17 @@ static int do_handle_out(cv4l_fd &fd, cv4l_queue &q, FILE *fin, cv4l_buffer *cap
 				buf.s_bytesused(cap->g_bytesused(j) - data_offset, j);
 				buf.s_data_offset(0, j);
 			} else if (q.g_memory() == V4L2_MEMORY_DMABUF) {
-				buf.s_bytesused(cap->g_bytesused(j), j);
+				__u32 bytesused = cap->g_bytesused(j);
+				/*
+				 * bytesused comes from the exported cap buffer
+				 * but the length of the out buffer might be smaller
+				 * so take the smaller of the two
+				 */
+				if (bytesused > buf.g_length(j))
+					bytesused = buf.g_length(j);
+				buf.s_bytesused(bytesused, j);
 				buf.s_data_offset(data_offset, j);
+				buf.s_fd(q.g_fd(buf.g_index(), j));
 			}
 		}
 	} else {
@@ -1531,10 +1642,10 @@ static int do_handle_out(cv4l_fd &fd, cv4l_queue &q, FILE *fin, cv4l_buffer *cap
 	if (fps_ts.has_fps()) {
 		unsigned dropped = fps_ts.dropped();
 
-		fprintf(stderr, " %.02f fps", fps_ts.fps());
+		stderr_info(" %.02f fps", fps_ts.fps());
 		if (dropped)
-			fprintf(stderr, ", dropped buffers: %u", dropped);
-		fprintf(stderr, "\n");
+			stderr_info(", dropped buffers: %u", dropped);
+		stderr_info("\n");
 	}
 	if (stopped)
 		return 0;
@@ -1599,7 +1710,7 @@ static int do_handle_out(cv4l_fd &fd, cv4l_queue &q, FILE *fin, cv4l_buffer *cap
 	tpg_update_mv_count(&tpg, V4L2_FIELD_HAS_T_OR_B(output_field));
 
 	if (!verbose)
-		fprintf(stderr, ">");
+		stderr_info(">");
 	fflush(stderr);
 
 	count++;
@@ -1607,8 +1718,11 @@ static int do_handle_out(cv4l_fd &fd, cv4l_queue &q, FILE *fin, cv4l_buffer *cap
 	if (ignore_count_skip)
 		return 0;
 
-	if (stream_sleep > 0 && count % stream_sleep == 0)
-		sleep(1);
+	if (need_sleep(count)) {
+		if (stream_sleep_mode & 2)
+			return QUEUE_OFF_ON;
+		do_sleep();
+	}
 
 	if (stream_skip) {
 		stream_skip--;
@@ -1648,7 +1762,7 @@ static int do_handle_out_to_in(cv4l_fd &out_fd, cv4l_fd &fd, cv4l_queue &out, cv
 
 static FILE *open_output_file(cv4l_fd &fd)
 {
-	FILE *fout = NULL;
+	FILE *fout = nullptr;
 
 #ifndef NO_STREAM_TO
 	if (file_to) {
@@ -1660,7 +1774,7 @@ static FILE *open_output_file(cv4l_fd &fd)
 		return fout;
 	}
 	if (!host_to)
-		return NULL;
+		return nullptr;
 
 	char *p = std::strchr(host_to, ':');
 	struct sockaddr_in serv_addr;
@@ -1673,7 +1787,7 @@ static FILE *open_output_file(cv4l_fd &fd)
 
 	aspect = fd.g_pixel_aspect(width, height);
 	if (p) {
-		host_port_to = strtoul(p + 1, 0L, 0);
+		host_port_to = strtoul(p + 1, nullptr, 0);
 		*p = '\0';
 	}
 	host_fd_to = socket(AF_INET, SOCK_STREAM, 0);
@@ -1682,7 +1796,7 @@ static FILE *open_output_file(cv4l_fd &fd)
 		std::exit(EXIT_SUCCESS);
 	}
 	server = gethostbyname(host_to);
-	if (server == NULL) {
+	if (server == nullptr) {
 		fprintf(stderr, "no such host %s\n", host_to);
 		std::exit(EXIT_SUCCESS);
 	}
@@ -1744,7 +1858,7 @@ static void streaming_set_cap(cv4l_fd &fd, cv4l_fd &exp_fd)
 	unsigned count;
 	bool eos;
 	bool source_change;
-	FILE *fout = NULL;
+	FILE *fout = nullptr;
 	cv4l_fmt fmt;
 
 	if (!(capabilities & (V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_VIDEO_CAPTURE_MPLANE |
@@ -1790,7 +1904,7 @@ recover:
 				while (fd.query_dv_timings(new_dv_timings))
 					sleep(1);
 				fd.s_dv_timings(new_dv_timings);
-				fprintf(stderr, "New timings found\n");
+				stderr_info("New timings found\n");
 			} else if (in.capabilities & V4L2_IN_CAP_STD) {
 				if (!fd.query_std(new_std))
 					fd.s_std(new_std);
@@ -1820,6 +1934,9 @@ recover:
 	if (q.obtain_bufs(&fd))
 		goto done;
 
+	fd.g_fmt(fmt);
+
+restart:
 	if (q.queue_all(&fd))
 		goto done;
 
@@ -1831,10 +1948,8 @@ recover:
 	fd.s_trace(0);
 	exp_fd.s_trace(0);
 
-	fd.g_fmt(fmt);
-
-	while (stream_sleep == 0)
-		sleep(100);
+	if (stream_sleep_count == 0)
+		do_sleep();
 
 	if (use_poll)
 		fcntl(fd.g_fd(), F_SETFL, fd_flags | O_NONBLOCK);
@@ -1849,17 +1964,17 @@ recover:
 		FD_SET(fd.g_fd(), &exception_fds);
 		FD_ZERO(&read_fds);
 		FD_SET(fd.g_fd(), &read_fds);
-		r = select(fd.g_fd() + 1, use_poll ? &read_fds : NULL, NULL, &exception_fds, &tv);
+		r = select(fd.g_fd() + 1, use_poll ? &read_fds : nullptr, nullptr, &exception_fds, &tv);
 
 		if (r == -1) {
 			if (EINTR == errno)
 				continue;
-			fprintf(stderr, "select error: %s\n",
+			stderr_info("select error: %s\n",
 					strerror(errno));
 			goto done;
 		}
 		if (use_poll && r == 0) {
-			fprintf(stderr, "select timeout\n");
+			stderr_info("select timeout\n");
 			goto done;
 		}
 
@@ -1871,14 +1986,14 @@ recover:
 				case V4L2_EVENT_SOURCE_CHANGE:
 					source_change = true;
 					if (!verbose)
-						fprintf(stderr, "\n");
-					fprintf(stderr, "SOURCE CHANGE EVENT\n");
+						stderr_info("\n");
+					stderr_info("SOURCE CHANGE EVENT\n");
 					break;
 				case V4L2_EVENT_EOS:
 					eos = true;
 					if (!verbose)
-						fprintf(stderr, "\n");
-					fprintf(stderr, "EOS EVENT\n");
+						stderr_info("\n");
+					stderr_info("EOS EVENT\n");
 					fflush(stderr);
 					break;
 				}
@@ -1886,8 +2001,15 @@ recover:
 		}
 
 		if (FD_ISSET(fd.g_fd(), &read_fds)) {
-			r = do_handle_cap(fd, q, fout, NULL,
+			r = do_handle_cap(fd, q, fout, nullptr,
 					  count, fps_ts, fmt, false);
+			if (r == QUEUE_OFF_ON) {
+				fd.streamoff();
+				fps_ts.reset();
+				do_sleep();
+				goto restart;
+			}
+
 			if (r < 0)
 				break;
 		}
@@ -1895,7 +2017,7 @@ recover:
 	}
 	fd.streamoff();
 	fcntl(fd.g_fd(), F_SETFL, fd_flags);
-	fprintf(stderr, "\n");
+	stderr_info("\n");
 
 	q.free(&fd);
 	tpg_free(&tpg);
@@ -1914,7 +2036,7 @@ done:
 
 static FILE *open_input_file(cv4l_fd &fd, __u32 type)
 {
-	FILE *fin = NULL;
+	FILE *fin = nullptr;
 
 	if (file_from) {
 		if (!strcmp(file_from, "-"))
@@ -1925,7 +2047,7 @@ static FILE *open_input_file(cv4l_fd &fd, __u32 type)
 		return fin;
 	}
 	if (!host_from)
-		return NULL;
+		return nullptr;
 
 	char *p = std::strchr(host_from, ':');
 	int listen_fd;
@@ -1933,7 +2055,7 @@ static FILE *open_input_file(cv4l_fd &fd, __u32 type)
 	struct sockaddr_in serv_addr = {}, cli_addr;
 
 	if (p) {
-		host_port_from = strtoul(p + 1, 0L, 0);
+		host_port_from = strtoul(p + 1, nullptr, 0);
 		*p = '\0';
 	}
 	listen_fd = socket(AF_INET, SOCK_STREAM, 0);
@@ -2048,7 +2170,7 @@ static void streaming_set_out(cv4l_fd &fd, cv4l_fd &exp_fd)
 	fps_timestamps fps_ts;
 	unsigned count = 0;
 	bool stopped = false;
-	FILE *fin = NULL;
+	FILE *fin = nullptr;
 	cv4l_fmt fmt;
 
 	fd.g_fmt(fmt);
@@ -2098,6 +2220,7 @@ static void streaming_set_out(cv4l_fd &fd, cv4l_fd &exp_fd)
 	if (q.obtain_bufs(&fd))
 		goto done;
 
+restart:
 	if (do_setup_out_buffers(fd, q, fin, true, false) < 0)
 		goto done;
 
@@ -2109,8 +2232,8 @@ static void streaming_set_out(cv4l_fd &fd, cv4l_fd &exp_fd)
 	fd.s_trace(0);
 	exp_fd.s_trace(0);
 
-	while (stream_sleep == 0)
-		sleep(100);
+	if (stream_sleep_count == 0)
+		do_sleep();
 
 	if (use_poll)
 		fcntl(fd.g_fd(), F_SETFL, fd_flags | O_NONBLOCK);
@@ -2129,23 +2252,29 @@ static void streaming_set_out(cv4l_fd &fd, cv4l_fd &exp_fd)
 			tv.tv_sec = 2;
 			tv.tv_usec = 0;
 
-			r = select(fd.g_fd() + 1, NULL, &fds, NULL, &tv);
+			r = select(fd.g_fd() + 1, nullptr, &fds, nullptr, &tv);
 
 			if (r == -1) {
 				if (EINTR == errno)
 					continue;
-				fprintf(stderr, "select error: %s\n",
+				stderr_info("select error: %s\n",
 					strerror(errno));
 				goto done;
 			}
 
 			if (r == 0) {
-				fprintf(stderr, "select timeout\n");
+				stderr_info("select timeout\n");
 				goto done;
 			}
 		}
-		r = do_handle_out(fd, q, fin, NULL,
+		r = do_handle_out(fd, q, fin, nullptr,
 				  count, fps_ts, fmt, stopped, false);
+		if (r == QUEUE_OFF_ON) {
+			fd.streamoff();
+			fps_ts.reset();
+			do_sleep();
+			goto restart;
+		}
 		if (r == QUEUE_STOPPED)
 			stopped = true;
 		if (r < 0)
@@ -2159,7 +2288,7 @@ static void streaming_set_out(cv4l_fd &fd, cv4l_fd &exp_fd)
 	}
 	fd.streamoff();
 	fcntl(fd.g_fd(), F_SETFL, fd_flags);
-	fprintf(stderr, "\n");
+	stderr_info("\n");
 
 	q.free(&fd);
 	tpg_free(&tpg);
@@ -2176,7 +2305,7 @@ enum stream_type {
 	OUT,
 };
 
-static int capture_setup(cv4l_fd &fd, cv4l_queue &in, cv4l_fd *exp_fd, cv4l_fmt *new_fmt = NULL)
+static int capture_setup(cv4l_fd &fd, cv4l_queue &in, cv4l_fd *exp_fd, cv4l_fmt *new_fmt = nullptr)
 {
 	if (fd.streamoff(in.g_type())) {
 		fprintf(stderr, "%s: fd.streamoff error\n", __func__);
@@ -2227,7 +2356,7 @@ static int capture_setup(cv4l_fd &fd, cv4l_queue &in, cv4l_fd *exp_fd, cv4l_fmt
 
 static void stateful_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 			 FILE *fin, FILE *fout, cv4l_fmt &fmt_in,
-			 cv4l_fmt &fmt_out, cv4l_fd *exp_fd_p)
+			 const cv4l_fmt &fmt_out, cv4l_fd *exp_fd_p)
 {
 	int fd_flags = fcntl(fd.g_fd(), F_GETFL);
 	fps_timestamps fps_ts[2];
@@ -2246,7 +2375,6 @@ static void stateful_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 
 	bool have_eos = subscribe_event(fd, V4L2_EVENT_EOS);
 	bool is_encoder = false;
-	enum codec_type codec_type = get_codec_type(fd);
 	bool ignore_count_skip = codec_type == ENCODER;
 
 	if (have_eos) {
@@ -2286,15 +2414,15 @@ static void stateful_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 	fps_ts[CAP].determine_field(fd.g_fd(), in.g_type());
 	fps_ts[OUT].determine_field(fd.g_fd(), out.g_type());
 
-	while (stream_sleep == 0)
-		sleep(100);
+	if (stream_sleep_count == 0)
+		do_sleep();
 
 	fcntl(fd.g_fd(), F_SETFL, fd_flags | O_NONBLOCK);
 
 	if (have_eos && stopped) {
 		if (!verbose)
-			fprintf(stderr, "\n");
-		fprintf(stderr, "STOP %sCODER\n", is_encoder ? "EN" : "DE");
+			stderr_info("\n");
+		stderr_info("STOP %sCODER\n", is_encoder ? "EN" : "DE");
 		if (is_encoder)
 			fd.encoder_cmd(enc_stop);
 		else
@@ -2327,42 +2455,42 @@ static void stateful_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 		if (r == -1) {
 			if (EINTR == errno)
 				continue;
-			fprintf(stderr, "select error: %s\n",
+			stderr_info("select error: %s\n",
 					strerror(errno));
 			return;
 		}
 		if (r == 0) {
 			if (!stopped)
-				fprintf(stderr, "select timeout");
-			fprintf(stderr, "\n");
+				stderr_info("select timeout");
+			stderr_info("\n");
 			return;
 		}
 
 		if (rd_fds && FD_ISSET(fd.g_fd(), rd_fds)) {
-			r = do_handle_cap(fd, in, fin, NULL,
+			r = do_handle_cap(fd, in, fin, nullptr,
 					  count[CAP], fps_ts[CAP], fmt_in,
 					  ignore_count_skip);
 			if (r == QUEUE_STOPPED)
 				break;
 			if (r < 0) {
-				rd_fds = NULL;
+				rd_fds = nullptr;
 				if (!have_eos) {
-					ex_fds = NULL;
+					ex_fds = nullptr;
 					break;
 				}
 			}
 		}
 
 		if (wr_fds && FD_ISSET(fd.g_fd(), wr_fds)) {
-			r = do_handle_out(fd, out, fout, NULL,
+			r = do_handle_out(fd, out, fout, nullptr,
 					  count[OUT], fps_ts[OUT], fmt_out, stopped,
 					  !ignore_count_skip);
 			if (r == QUEUE_STOPPED) {
 				stopped = true;
 				if (have_eos) {
 					if (!verbose)
-						fprintf(stderr, "\n");
-					fprintf(stderr, "STOP %sCODER\n", is_encoder ? "EN" : "DE");
+						stderr_info("\n");
+					stderr_info("STOP %sCODER\n", is_encoder ? "EN" : "DE");
 					if (is_encoder)
 						fd.encoder_cmd(enc_stop);
 					else
@@ -2378,15 +2506,15 @@ static void stateful_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 
 			while (!fd.dqevent(ev)) {
 				if (ev.type == V4L2_EVENT_EOS) {
-					wr_fds = NULL;
+					wr_fds = nullptr;
 					if (!verbose)
-						fprintf(stderr, "\n");
-					fprintf(stderr, "EOS EVENT\n");
+						stderr_info("\n");
+					stderr_info("EOS EVENT\n");
 					fflush(stderr);
 				} else if (ev.type == V4L2_EVENT_SOURCE_CHANGE) {
 					if (!verbose)
-						fprintf(stderr, "\n");
-					fprintf(stderr, "SOURCE CHANGE EVENT\n");
+						stderr_info("\n");
+					stderr_info("SOURCE CHANGE EVENT\n");
 					in_source_change_event = true;
 
 					/*
@@ -2416,7 +2544,7 @@ static void stateful_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 	}
 
 	fcntl(fd.g_fd(), F_SETFL, fd_flags);
-	fprintf(stderr, "\n");
+	stderr_info("\n");
 
 	fd.streamoff(in.g_type());
 	fd.streamoff(out.g_type());
@@ -2427,7 +2555,7 @@ static void stateful_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 
 static void stateless_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 			  FILE *fin, FILE *fout, cv4l_fmt &fmt_in,
-			  cv4l_fmt &fmt_out, cv4l_fd *exp_fd_p)
+			  const cv4l_fmt &fmt_out, cv4l_fd *exp_fd_p)
 {
 	fps_timestamps fps_ts[2];
 	unsigned count[2] = { 0, 0 };
@@ -2488,7 +2616,7 @@ static void stateless_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 		FD_ZERO(&except_fds);
 		FD_SET(req_fd, &except_fds);
 
-		int rc = select(req_fd + 1, NULL, NULL, &except_fds, &tv);
+		int rc = select(req_fd + 1, nullptr, nullptr, &except_fds, &tv);
 
 		if (rc == 0) {
 			fprintf(stderr, "Timeout when waiting for media request\n");
@@ -2505,8 +2633,8 @@ static void stateless_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 		 * is not needed anymore as a reference frame
 		 */
 		if (queue_lst_buf) {
-			if (fd.qbuf(last_in_buf) < 0) {
-				fprintf(stderr, "%s: qbuf failed\n", __func__);
+			if (fd.qbuf(last_in_buf)) {
+				stderr_info("%s: qbuf failed\n", __func__);
 				return;
 			}
 		}
@@ -2515,10 +2643,10 @@ static void stateless_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 		 * fin is not sent to do_handle_cap since the capture buf is
 		 * written to the file in current function
 		 */
-		rc = do_handle_cap(fd, in, NULL, &buf_idx, count[CAP],
+		rc = do_handle_cap(fd, in, nullptr, &buf_idx, count[CAP],
 				   fps_ts[CAP], fmt_in, false);
 		if (rc && rc != QUEUE_STOPPED) {
-			fprintf(stderr, "%s: do_handle_cap err\n", __func__);
+			stderr_info("%s: do_handle_cap err\n", __func__);
 			return;
 		}
 		/*
@@ -2528,7 +2656,7 @@ static void stateless_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 		 * all the future capture buffers.
 		 */
 		if (buf_idx == -1) {
-			fprintf(stderr, "%s: frame returned with error\n", __func__);
+			stderr_info("%s: frame returned with error\n", __func__);
 			last_fwht_bf_ts	= 0;
 		} else {
 			cv4l_buffer cap_buf(in, index);
@@ -2555,12 +2683,12 @@ static void stateless_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 			return;
 
 		if (!stopped) {
-			rc = do_handle_out(fd, out, fout, NULL, count[OUT],
+			rc = do_handle_out(fd, out, fout, nullptr, count[OUT],
 					   fps_ts[OUT], fmt_out, false, true);
 			if (rc) {
 				stopped = true;
 				if (rc != QUEUE_STOPPED)
-					fprintf(stderr, "%s: output stream ended\n", __func__);
+					stderr_info("%s: output stream ended\n", __func__);
 				close(req_fd);
 				fwht_reqs[index].fd = -1;
 			}
@@ -2569,7 +2697,7 @@ static void stateless_m2m(cv4l_fd &fd, cv4l_queue &in, cv4l_queue &out,
 	}
 
 	fcntl(fd.g_fd(), F_SETFL, fd_flags);
-	fprintf(stderr, "\n");
+	stderr_info("\n");
 
 	fd.streamoff(in.g_type());
 	fd.streamoff(out.g_type());
@@ -2583,8 +2711,8 @@ static void streaming_set_m2m(cv4l_fd &fd, cv4l_fd &exp_fd)
 	cv4l_queue in(fd.g_type(), memory);
 	cv4l_queue out(v4l_type_invert(fd.g_type()), out_memory);
 	cv4l_queue exp_q(exp_fd.g_type(), V4L2_MEMORY_MMAP);
-	cv4l_fd *exp_fd_p = NULL;
-	FILE *file[2] = {NULL, NULL};
+	cv4l_fd *exp_fd_p = nullptr;
+	FILE *file[2] = {nullptr, nullptr};
 	cv4l_fmt fmt[2];
 
 	fd.g_fmt(fmt[OUT], out.g_type());
@@ -2645,12 +2773,12 @@ static void streaming_set_cap2out(cv4l_fd &fd, cv4l_fd &out_fd)
 	cv4l_queue out(out_type, out_memory);
 	fps_timestamps fps_ts[2];
 	unsigned count[2] = { 0, 0 };
-	FILE *file[2] = {NULL, NULL};
+	FILE *file[2] = {nullptr, nullptr};
 	fd_set fds;
 	unsigned cnt = 0;
 	cv4l_fmt fmt[2];
 
-	fd.g_fmt(fmt[OUT], out.g_type());
+	out_fd.g_fmt(fmt[OUT], out.g_type());
 	fd.g_fmt(fmt[CAP], in.g_type());
 	if (!(capabilities & (V4L2_CAP_VIDEO_CAPTURE |
 			      V4L2_CAP_VIDEO_CAPTURE_MPLANE |
@@ -2724,14 +2852,24 @@ static void streaming_set_cap2out(cv4l_fd &fd, cv4l_fd &out_fd)
 		fprintf(stderr, "mismatch between number of planes\n");
 		goto done;
 	}
-
-	if (in.obtain_bufs(&fd) ||
-	    in.queue_all(&fd) ||
-	    do_setup_out_buffers(out_fd, out, file[OUT], false, false) == QUEUE_ERROR)
+	if (in.obtain_bufs(&fd)) {
+		fprintf(stderr, "%s: in.obtain_bufs failed\n", __func__);
 		goto done;
+	}
+
+	if (in.queue_all(&fd)) {
+		fprintf(stderr, "%s: in.queue_all failed\n", __func__);
+		goto done;
+	}
+
+
+	if (do_setup_out_buffers(out_fd, out, file[OUT], false, false) == QUEUE_ERROR) {
+		fprintf(stderr, "%s: do_setup_out_buffers failed\n", __func__);
+		goto done;
+	}
 
 	fps_ts[CAP].determine_field(fd.g_fd(), in.g_type());
-	fps_ts[OUT].determine_field(fd.g_fd(), out.g_type());
+	fps_ts[OUT].determine_field(out_fd.g_fd(), out.g_type());
 
 	if (fd.streamon() || out_fd.streamon())
 		goto done;
@@ -2739,8 +2877,8 @@ static void streaming_set_cap2out(cv4l_fd &fd, cv4l_fd &out_fd)
 	fd.s_trace(0);
 	out_fd.s_trace(0);
 
-	while (stream_sleep == 0)
-		sleep(100);
+	if (stream_sleep_count == 0)
+		do_sleep();
 
 	if (use_poll)
 		fcntl(fd.g_fd(), F_SETFL, fd_flags | O_NONBLOCK);
@@ -2753,17 +2891,17 @@ static void streaming_set_cap2out(cv4l_fd &fd, cv4l_fd &out_fd)
 		FD_SET(fd.g_fd(), &fds);
 
 		if (use_poll)
-			r = select(fd.g_fd() + 1, &fds, NULL, NULL, &tv);
+			r = select(fd.g_fd() + 1, &fds, nullptr, nullptr, &tv);
 
 		if (r == -1) {
 			if (EINTR == errno)
 				continue;
-			fprintf(stderr, "select error: %s\n",
+			stderr_info("select error: %s\n",
 					strerror(errno));
 			goto done;
 		}
 		if (use_poll && r == 0) {
-			fprintf(stderr, "select timeout\n");
+			stderr_info("select timeout\n");
 			goto done;
 		}
 
@@ -2773,7 +2911,7 @@ static void streaming_set_cap2out(cv4l_fd &fd, cv4l_fd &out_fd)
 			r = do_handle_cap(fd, in, file[CAP], &index,
 					  count[CAP], fps_ts[CAP], fmt[CAP], true);
 			if (r)
-				fprintf(stderr, "handle cap %d\n", r);
+				stderr_info("handle cap %d\n", r);
 			if (!r) {
 				cv4l_buffer buf(in, index);
 
@@ -2784,11 +2922,11 @@ static void streaming_set_cap2out(cv4l_fd &fd, cv4l_fd &out_fd)
 						  false, false);
 			}
 			if (r)
-				fprintf(stderr, "handle out %d\n", r);
+				stderr_info("handle out %d\n", r);
 			if (!r && cnt++ > 1)
 				r = do_handle_out_to_in(out_fd, fd, out, in);
 			if (r)
-				fprintf(stderr, "handle out2in %d\n", r);
+				stderr_info("handle out2in %d\n", r);
 			if (r < 0) {
 				fd.streamoff();
 				out_fd.streamoff();
@@ -2799,7 +2937,7 @@ static void streaming_set_cap2out(cv4l_fd &fd, cv4l_fd &out_fd)
 
 done:
 	fcntl(fd.g_fd(), F_SETFL, fd_flags);
-	fprintf(stderr, "\n");
+	stderr_info("\n");
 
 	if (options[OptStreamDmaBuf])
 		out.close_exported_fds();
@@ -2842,6 +2980,7 @@ void streaming_set(cv4l_fd &fd, cv4l_fd &out_fd, cv4l_fd &exp_fd)
 
 	get_cap_compose_rect(fd);
 	get_out_crop_rect(fd);
+	get_codec_type(fd);
 
 	if (do_cap && do_out && out_fd.g_fd() < 0)
 		streaming_set_m2m(fd, exp_fd);
