@@ -66,6 +66,7 @@ static struct option long_options[] = {
 	{"try-fmt-video-out", required_argument, nullptr, OptTryVideoOutFormat},
 	{"get-routing", no_argument, 0, OptGetRouting},
 	{"set-routing", required_argument, 0, OptSetRouting},
+	{"try-routing", required_argument, 0, OptTryRouting},
 	{"help", no_argument, nullptr, OptHelp},
 	{"help-tuner", no_argument, nullptr, OptHelpTuner},
 	{"help-io", no_argument, nullptr, OptHelpIO},
@@ -221,7 +222,7 @@ static struct option long_options[] = {
 	{"get-edid", optional_argument, nullptr, OptGetEdid},
 	{"info-edid", optional_argument, nullptr, OptInfoEdid},
 	{"show-edid", required_argument, nullptr, OptShowEdid},
-	{"fix-edid-checksums", no_argument, nullptr, OptFixEdidChecksums},
+	{"keep-edid-checksums", no_argument, nullptr, OptKeepEdidChecksums},
 	{"tuner-index", required_argument, nullptr, OptTunerIndex},
 	{"list-buffers", no_argument, nullptr, OptListBuffers},
 	{"list-buffers-out", no_argument, nullptr, OptListBuffersOut},
@@ -1138,6 +1139,7 @@ int main(int argc, char **argv)
 	const char *export_device = nullptr;
 	struct v4l2_capability vcap = {};
 	struct v4l2_subdev_capability subdevcap = {};
+	struct v4l2_subdev_client_capability subdevclientcap = {};
 	std::vector<event> events;
 	unsigned secs = 0;
 	char short_options[26 * 2 * 3 + 1];
@@ -1274,7 +1276,7 @@ int main(int argc, char **argv)
 			common_usage();
 			return 1;
 		default:
-			common_cmd(media_bus_info, ch, optarg);
+			common_cmd(ch, optarg);
 			tuner_cmd(ch, optarg);
 			io_cmd(ch, optarg);
 			stds_cmd(ch, optarg);
@@ -1354,6 +1356,9 @@ int main(int argc, char **argv)
 		// This ioctl was introduced in kernel 5.10, so don't
 		// exit if this ioctl returns an error.
 		doioctl(fd, VIDIOC_SUBDEV_QUERYCAP, &subdevcap);
+		subdevclientcap.capabilities = ~0ULL;
+		if (doioctl(fd, VIDIOC_SUBDEV_S_CLIENT_CAP, &subdevclientcap))
+			subdevclientcap.capabilities = 0ULL;
 	}
 	if (!is_subdev) {
 		capabilities = vcap.capabilities;
@@ -1460,10 +1465,11 @@ int main(int argc, char **argv)
 	if (options[OptGetDriverInfo]) {
 		printf("Driver Info%s:\n",
 				options[OptUseWrapper] ? " (using libv4l2)" : "");
-		if (is_subdev)
-			v4l2_info_subdev_capability(subdevcap);
-		else
+		if (is_subdev) {
+			v4l2_info_subdev_capability(subdevcap, subdevclientcap);
+		} else {
 			v4l2_info_capability(vcap);
+		}
 	}
 	if (options[OptGetDriverInfo] && media_fd >= 0)
 		mi_media_info_for_fd(media_fd, fd);
@@ -1504,7 +1510,7 @@ int main(int argc, char **argv)
 
 	/* List options */
 
-	common_list(c_fd);
+	common_list(media_bus_info, c_fd);
 	io_list(c_fd);
 	stds_list(c_fd);
 	vidcap_list(c_fd);
